import React, { Component } from 'react';


// Top-level component that lets the user add/complete items on a todo list
export class TodoApp extends Component {

  constructor(props) {
    super(props);
    this.state = {items: [], newName: ""};
  }

  render = () => {
    // Return a UI with all the items and elements that allow them to add a new
    // item with a name of their choice.
    return (
      <div>
        <h2>To-Do List</h2>
        {this.renderItems()}
        <p>Check the item to mark it completed.</p>
        <p>New item:
          <input type="text" value={this.state.newName}
              onChange={this.doNewNameChange} />
          <button type="button"
              onClick={this.doAddClick}>Add</button>
        </p>
      </div>);
  }

  // Returns a div for each item in the todo list, with each div containing a
  // label with the name of the item and a checkbox for marking it completed.
  renderItems = () => {
    const items = [];
    for (let i = 0; i < this.state.items.length; i++) {
      if (this.state.items[i].completed) {
        items.push(
          <div key={i}>
            <input type="checkbox" id={"check" + i} checked={true} readOnly={true} />
            <label htmlFor={"check" + i}><s>{this.state.items[i].name}</s></label>
          </div>);
      } else {
        items.push(
          <div key={i}>
            <input type="checkbox" id={"check" + i} checked={false}
                onChange={evt => this.doItemClick(evt, i)} />
            <label htmlFor={"check" + i}>{this.state.items[i].name}</label>
          </div>);
      }
    }
    return items;
  };

  // Called each time the text in the new item name field is changed.
  doNewNameChange = (evt) => {
    this.setState({newName: evt.target.value});
  }

  // Called when the user clicks on the button to add the new item.
  doAddClick = (evt) => {
    // Ignore the request if the user hasn't entered a name.
    // (Would be better to disable the button in this case.)
    const name = this.state.newName.trim();
    if (name.length == 0)
      return;

    const items = this.state.items.concat([{name: name, completed: false}]);
    this.setState({items: items, newName: ""});
  }

  // Called when the user checks the box next to an uncompleted item. The
  // second parameter is the index of that item in the list.
  doItemClick = (evt, index) => {
    const item = this.state.items[index];

    // Note: we cannot mutate the list. We have to create a new one.
    const items = this.state.items.slice(0, index)
        .concat([{name: item.name, completed: true}])
        .concat(this.state.items.slice(index + 1));
    this.setState({items: items});

    setTimeout(() => this.doItemTimeout(index), 1000); 
  }

  // Called after an item has been removed for 1 second.
  doItemTimeout = (index) => {
    const item = this.state.items[index];

    // Note: we cannot mutate the list. We have to create a new one.
    const items = this.state.items.slice(0, index)
        .concat(this.state.items.slice(index + 1));
    this.setState({items: items});
  }
}
